/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package cz.fidentis.analyst.procrustes;

import cz.fidentis.analyst.icp.Quaternion;
import javax.vecmath.Vector3d;
import org.ejml.simple.SimpleMatrix;

/**
 * Class used to store information about transformation that needs to be applied
 * to a second face when computing procrustes analysis
 *
 * @author Jakub Kolman
 */
public class ProcrustesTransformation {

    private double scale;
    private Vector3d adjustment;
    private SimpleMatrix rotationMatrix;
    private ProcrustesAnalysisFaceModel faceModel;

    /**
     * constructor
     *
     * @param scale determines by how much should be faced enlarged or minimized
     * @param adjustment determines how should be face moved to make faces
     * superimposed considering feature points
     * @param rotationMatrix determines how to rotate face
     * @param updatedFaceModel holds information about adjusted face
     */
    public ProcrustesTransformation(
            double scale,
            Vector3d adjustment,
            SimpleMatrix rotationMatrix,
            ProcrustesAnalysisFaceModel updatedFaceModel) {
        this.scale = scale;
        this.adjustment = adjustment;
        this.rotationMatrix = rotationMatrix;
        this.faceModel = updatedFaceModel;
    }

    /**
     * Constructor (for more info take a look on 
     * {@link #ProcrustesTransformation(double, javax.vecmath.Vector3d, org.ejml.simple.SimpleMatrix, cz.fidentis.analyst.procrustes.ProcrustesAnalysisFaceModel) }
     *
     * @param adjustment
     * @param rotationMatrix
     */
    public ProcrustesTransformation(Vector3d adjustment, SimpleMatrix rotationMatrix) {
        this.scale = 1;
        this.adjustment = adjustment;
        this.rotationMatrix = rotationMatrix;
    }

    /**
     * Empty constructor. To set values later attribute setters can be used.
     * (for more info take a look on 
     * {@link #ProcrustesTransformation(double, javax.vecmath.Vector3d, org.ejml.simple.SimpleMatrix, cz.fidentis.analyst.procrustes.ProcrustesAnalysisFaceModel) }
     *
     * @param adjustment
     * @param rotationMatrix
     */
    public ProcrustesTransformation() {
        this.scale = 1;
        this.adjustment = null;
        this.rotationMatrix = null;
    }

    public double getScale() {
        return this.scale;
    }

    public Vector3d getAdjustment() {
        return this.adjustment;
    }

    public SimpleMatrix getRotationMatrix() {
        return this.rotationMatrix;
    }

    public ProcrustesAnalysisFaceModel getFaceModel() {
        return faceModel;
    }

    public void setScale(double scale) {
        this.scale = scale;
    }

    public void setFaceModel(ProcrustesAnalysisFaceModel faceModel) {
        this.faceModel = faceModel;
    }

    /**
     * Calculates quaternion from rotation matrix.
     *
     * @return quaternion with rotation values
     */
    public Quaternion getRotationMatrixAsQuaternion() {
        if (rotationMatrix != null) {
            double w = Math.sqrt(1.0 + rotationMatrix.get(0, 0) + rotationMatrix.get(1, 1) + rotationMatrix.get(2, 2)) / 2.0;
            double w4 = (4.0 * w);
            double x = (rotationMatrix.get(2, 1) - rotationMatrix.get(1, 2)) / w4;
            double y = (rotationMatrix.get(0, 2) - rotationMatrix.get(2, 0)) / w4;
            double z = (rotationMatrix.get(1, 0) - rotationMatrix.get(0, 1)) / w4;

            return new Quaternion(x, y, z, w);
        } else {
            return null;
        }
    }

    public void setAdjustment(Vector3d adjustment) {
        this.adjustment = adjustment;
    }

    public void setRotationMatrix(SimpleMatrix rotationMatrix) {
        this.rotationMatrix = rotationMatrix;
    }
}
